@@ -58,7 +58,8 @@ typedef enum /* Dwarf_Check_Categories */ {
         won't really affect client code.  The 'harmless' errors
         are reported and otherwise ignored.  It is difficult to report
         the error when the error is noticed by libdwarf, the error
-        is reported at a later time.
+        is reported at a later time. See
+        dwarf_set_harmless_errors() to turn off such error checks.
         The other errors dwarfdump reports are also generally harmless
         but are detected by dwarfdump so it's possble to report the
         error as soon as the error is discovered. */
@@ -181,6 +182,10 @@ struct glflags_s {
     Dwarf_Bool gf_generic_1200_regs;
     Dwarf_Bool gf_suppress_check_extensions_tables;
 
+    /* a call to libdwarf will speed up the library
+        See dwarf_set_harmless_errors_enabled() */
+    int        gf_suppress_harmless;
+
     /* The following tells libdwarf not to check for duplicated
         attributes if TRUE. */
     Dwarf_Bool gf_no_check_duplicated_attributes;
@@ -205,8 +210,10 @@ struct glflags_s {
 
     Dwarf_Bool gf_check_names;
 
-    /* During '-k' mode, display errors */
-    Dwarf_Bool gf_check_verbose_mode;
+    /*  During '-k' mode, display errors if non-zero.
+        If > 1 then display DWARF CHECK errors
+        in full detail from  dd_check_attr_encoding.c */
+    int        gf_check_verbose_mode;
 
     Dwarf_Bool gf_check_frames;
     Dwarf_Bool gf_check_frames_extended; /* Extensive frames check */
