@@ -1082,6 +1082,7 @@ print_one_fde(Dwarf_Debug dbg,
 
     fde = fde_data[fde_index];
     fres = dwarf_get_fde_range(fde,
+
         &low_pc, &func_length,
         &fde_bytes,
         &fde_bytes_length,
@@ -2087,7 +2088,7 @@ print_frame_inst_bytes(Dwarf_Debug dbg,
         printf(
             " Warning: Final FDE operator is useless "
             "but not an error. %s\n",
-            get_CFA_name(lastop,TRUE));
+            get_CFA_name(lastop));
     }
 }
 
