@@ -31,6 +31,7 @@
 #include <config.h>
 
 #include <stddef.h> /* NULL size_t */
+#include <stdio.h> /* debug printf */
 
 #if defined(_WIN32) && defined(HAVE_STDAFX_H)
 #include "stdafx.h"
@@ -38,6 +39,7 @@
 
 #include "dwarf.h"
 #include "libdwarf.h"
+#include "dwarf_local_malloc.h"
 #include "libdwarf_private.h"
 #include "dwarf_base_types.h"
 #include "dwarf_opaque.h"
@@ -77,7 +79,7 @@ free_aranges_chain(Dwarf_Debug dbg, Dwarf_Chain head)
     callers will not repeat the error often or mind the leaks.
 */
 static int
-dwarf_get_aranges_list(Dwarf_Debug dbg,
+_dwarf_get_aranges_list(Dwarf_Debug dbg,
     Dwarf_Chain  * chain_out,
     Dwarf_Signed * chain_count_out,
     Dwarf_Error  * error)
@@ -99,7 +101,7 @@ dwarf_get_aranges_list(Dwarf_Debug dbg,
     /*  Size in bytes of addresses in target. */
     Dwarf_Small address_size = 0;
     /*  Size in bytes of segment offsets in target. */
-    Dwarf_Small segment_size = 0;
+    Dwarf_Small segment_sel_size = 0;
     /*  Count of total number of aranges. */
     Dwarf_Signed arange_count = 0;
     Dwarf_Arange arange = 0;
@@ -123,7 +125,7 @@ dwarf_get_aranges_list(Dwarf_Debug dbg,
             This is local length, which begins just
             after the length field itself. */
         Dwarf_Unsigned area_length = 0;
-        Dwarf_Small remainder = 0;
+        Dwarf_Unsigned remainder = 0;
         Dwarf_Unsigned range_entry_size = 0;
         int local_length_size;
         int local_extension_size = 0;
@@ -146,7 +148,9 @@ dwarf_get_aranges_list(Dwarf_Debug dbg,
         /*  arange_ptr has been incremented appropriately past
             the length field by READ_AREA_LENGTH. */
 
-        if (area_length >  dbg->de_debug_aranges.dss_size) {
+        if (area_length >  dbg->de_debug_aranges.dss_size ||
+            (area_length +local_length_size+local_extension_size)
+            > dbg->de_debug_aranges.dss_size ) {
             free_aranges_chain(dbg,head_chain);
             _dwarf_error(dbg, error,DW_DLE_ARANGES_HEADER_ERROR);
             return DW_DLV_ERROR;
@@ -235,13 +239,19 @@ dwarf_get_aranges_list(Dwarf_Debug dbg,
             return DW_DLV_ERROR;
         }
 
-        /*  Even DWARF2 had a segment_size field here, meaning
-            size in bytes of a segment descriptor on the target
-            system. */
-        segment_size = *(Dwarf_Small *) arange_ptr;
-        if (segment_size > sizeof(Dwarf_Addr)) {
+        /*  Even DWARF2 had a segment_sel_size field here,
+            meaning
+            size in bytes of a segment selector/descriptor
+            on the target system.
+            In reality it is unlikely any non-zero
+            value will work sensibly for the user.  */
+        segment_sel_size = *(Dwarf_Small *) arange_ptr;
+        if (segment_sel_size > 0) {
             free_aranges_chain(dbg,head_chain);
-            _dwarf_error(dbg, error, DW_DLE_SEGMENT_SIZE_BAD);
+            _dwarf_error_string(dbg, error,
+                DW_DLE_SEGMENT_SIZE_BAD,
+                "DW_DLE_SEGMENT_SIZE_BAD: "
+                "segment selector size > 0 is not supported");
             return DW_DLV_ERROR;
         }
         arange_ptr = arange_ptr + sizeof(Dwarf_Small);
@@ -252,11 +262,11 @@ dwarf_get_aranges_list(Dwarf_Debug dbg,
             _dwarf_error(dbg, error, DW_DLE_ARANGE_OFFSET_BAD);
             return DW_DLV_ERROR;
         }
-        range_entry_size = 2*address_size + segment_size;
+        range_entry_size = 2*address_size + segment_sel_size;
         /*  Round arange_ptr offset to next multiple of
             address_size. */
-        remainder = (Dwarf_Unsigned) (arange_ptr - header_ptr) %
-            (range_entry_size);
+        remainder = (Dwarf_Unsigned) ((arange_ptr - header_ptr) %
+            (range_entry_size));
         if (remainder != 0) {
             arange_ptr = arange_ptr + (2 * address_size) - remainder;
         }
@@ -269,25 +279,15 @@ dwarf_get_aranges_list(Dwarf_Debug dbg,
                 read is a segment selector (new in DWARF4).
                 The version number DID NOT CHANGE from 2, which
                 is quite surprising.
-                Also surprising since the segment_size
+                Also surprising since the segment_sel_size
                 was always there
                 in the table header! */
-            /*  We want to test cu_version here but
-                currently with no way to do that.
-                So we just hope no one using
+            /*  We want to test cu_version here
+                and segment_sel_size, but
+                currently with no way segment_sel_size
+                can be other than zero.
+                We just hope no one using
                 segment_selectors, really. FIXME */
-            if (segment_size) {
-                /*  Only applies if cu_version >= 4. */
-                res = _dwarf_read_unaligned_ck_wrapper(dbg,
-                    &segment_selector,
-                    arange_ptr,segment_size,end_this_arange,error);
-                if (res != DW_DLV_OK) {
-                    free_aranges_chain(dbg,head_chain);
-                    return res;
-                }
-                arange_ptr += address_size;
-            }
-
             res = _dwarf_read_unaligned_ck_wrapper(dbg,&range_address,
                 arange_ptr,address_size,end_this_arange,error);
             if (res != DW_DLV_OK) {
@@ -319,7 +319,8 @@ dwarf_get_aranges_list(Dwarf_Debug dbg,
                 }
 
                 arange->ar_segment_selector = segment_selector;
-                arange->ar_segment_selector_size = segment_size;
+                arange->ar_segment_selector_size =
+                    segment_sel_size;
                 arange->ar_address = range_address;
                 arange->ar_length = range_length;
                 arange->ar_info_offset = info_offset;
@@ -353,7 +354,8 @@ dwarf_get_aranges_list(Dwarf_Debug dbg,
         /*  A compiler could emit some padding bytes here. dwarf2/3
             (dwarf4 sec 7.20) does not clearly make extra padding
             bytes illegal. */
-        if (end_this_arange < arange_ptr) {
+        if (end_this_arange < arange_ptr &&
+            dbg->de_harmless_errors_on) {
             Dwarf_Unsigned pad_count = arange_ptr - end_this_arange;
             Dwarf_Unsigned offset = arange_ptr - arange_ptr_start;
             dwarfstring aramsg;
@@ -416,11 +418,7 @@ dwarf_get_aranges(Dwarf_Debug dbg,
 
     /* ***** BEGIN CODE ***** */
 
-    if (dbg == NULL) {
-        _dwarf_error(NULL, error, DW_DLE_DBG_NULL);
-        return DW_DLV_ERROR;
-    }
-
+    CHECK_DBG(dbg,error,"dwarf_get_aranges()");
     res = _dwarf_load_section(dbg, &dbg->de_debug_aranges, error);
     if (res != DW_DLV_OK) {
         return res;
@@ -432,7 +430,8 @@ dwarf_get_aranges(Dwarf_Debug dbg,
         return res;
     }
 
-    res = dwarf_get_aranges_list(dbg,&head_chain,&arange_count,error);
+    res = _dwarf_get_aranges_list(dbg,&head_chain,
+        &arange_count,error);
     if (res != DW_DLV_OK) {
         free_aranges_chain(dbg,head_chain);
         return res;
@@ -463,6 +462,7 @@ dwarf_get_aranges(Dwarf_Debug dbg,
     return DW_DLV_OK;
 }
 
+#if 0 /* _dwarf_get_aranges_addr_offsets Unused */
 /*
     This function returns DW_DLV_OK if it succeeds
     and DW_DLV_ERR or DW_DLV_OK otherwise.
@@ -498,7 +498,7 @@ _dwarf_get_aranges_addr_offsets(Dwarf_Debug dbg,
     if (error != NULL)
         *error = NULL;
 
-    if (dbg == NULL) {
+    if (IS_INVALID_DBG(dbg)) {
         _dwarf_error(NULL, error, DW_DLE_DBG_NULL);
         return DW_DLV_ERROR;
     }
@@ -513,7 +513,8 @@ _dwarf_get_aranges_addr_offsets(Dwarf_Debug dbg,
     if (res != DW_DLV_OK) {
         return res;
     }
-    res = dwarf_get_aranges_list(dbg,&head_chain,&arange_count,error);
+    res = _dwarf_get_aranges_list(dbg,&head_chain,
+        &arange_count,error);
     if (res != DW_DLV_OK) {
         return res;
     }
@@ -558,6 +559,7 @@ _dwarf_get_aranges_addr_offsets(Dwarf_Debug dbg,
     *addrs = arange_addrs;
     return DW_DLV_OK;
 }
+#endif /* 0 */
 
 /*
     This function takes a pointer to a block
@@ -629,7 +631,7 @@ dwarf_get_cu_die_offset(Dwarf_Arange arange,
         }
     }
     cres = _dwarf_length_of_cu_header(dbg, offset,
-        true, &headerlen,error);
+        TRUE, &headerlen,error);
     if (cres != DW_DLV_OK) {
         return cres;
     }
@@ -671,16 +673,20 @@ dwarf_get_arange_cu_header_offset(Dwarf_Arange arange,
 
 /*
     This function takes a Dwarf_Arange, and returns
-    true if it is not NULL.  It also stores the start
+    TRUE if it is not NULL.  It also stores the start
     address of the range in *start, the length of the
     range in *length, and the offset of the first die
     in the compilation-unit in *cu_die_offset.  It
-    returns false on error.
+    returns FALSE on error.
     If cu_die_offset returned ensures .debug_info loaded so
     the cu_die_offset is meaningful.
 
     New for DWARF4, entries may have segment information.
-    *segment is only meaningful if *segment_entry_size is non-zero. */
+    *segment is only meaningful
+    if *segment_entry_size is non-zero.
+    But segment_selectors are not fully defined so
+    a non-zero segment_entry_size is not actually
+    usable. */
 int
 dwarf_get_arange_info_b(Dwarf_Arange arange,
     Dwarf_Unsigned*  segment,
@@ -719,7 +725,7 @@ dwarf_get_arange_info_b(Dwarf_Arange arange,
             }
         }
         cres = _dwarf_length_of_cu_header(dbg, offset,
-            true, &headerlen,error);
+            TRUE, &headerlen,error);
         if (cres != DW_DLV_OK) {
             return cres;
         }
