@@ -30,6 +30,7 @@
 
 #include <stdlib.h> /* calloc() free() malloc() */
 #include <string.h> /* memset() strcat() strlen() */
+#include <stdio.h> /* debugging */
 
 #if defined(_WIN32) && defined(HAVE_STDAFX_H)
 #include "stdafx.h"
@@ -37,6 +38,7 @@
 
 #include "dwarf.h"
 #include "libdwarf.h"
+#include "dwarf_local_malloc.h"
 #include "libdwarf_private.h"
 #include "dwarf_base_types.h"
 #include "dwarf_safe_strcpy.h"
@@ -46,6 +48,7 @@
 #include "dwarf_util.h"
 #include "dwarf_macro5.h"
 #include "dwarf_string.h"
+#include "dwarf_str_offsets.h"
 
 #define MC_SENTINEL  0xada
 
@@ -219,7 +222,7 @@ _dwarf_skim_forms(Dwarf_Debug dbg,
             }
             v = strlen((char *) mdata) + 1;
             totallen += v;
-            mdata += v;
+            mdata    += v;
             }
             break;
         case DW_FORM_block:
@@ -312,7 +315,12 @@ is_defundef(unsigned op)
 #endif /*0*/
 
 /*  On first call (for this macro_context),
-    build_ops_array is FALSE. On second,
+    build_ops_array is FALSE.
+    We just calculate
+        macro_context->mc_macro_ops_count
+        macro_context->mc_ops_data_length
+        macro_context->mc_total_length
+    On second,
     it is TRUE and we know the count so we allocate and fill in
     the ops array. */
 static int
@@ -331,8 +339,8 @@ _dwarf_get_macro_ops_count_internal(Dwarf_Macro_Context macro_context,
     int res = 0;
 
     dbg = macro_context->mc_dbg;
+    known_ops_count = macro_context->mc_macro_ops_count;
     if (build_ops_array) {
-        known_ops_count = macro_context->mc_macro_ops_count;
         opsarray = (struct Dwarf_Macro_Operator_s *)
             calloc(known_ops_count,
                 sizeof(struct Dwarf_Macro_Operator_s));
@@ -360,6 +368,19 @@ _dwarf_get_macro_ops_count_internal(Dwarf_Macro_Context macro_context,
                 Normally we don't see this, the end operator
                 signals end. */
             opslen = mdata - macro_context->mc_macro_ops;
+#if 0 /* Dropping a counterproductive test. */
+            /*  This test fails when there are corrupted FORms
+                so we lose a really useful and precise error
+                message and get this useless message. */
+            if (known_ops_count != opcount) {
+                _dwarf_error_string(dbg, error,
+                    DW_DLE_MACRO_OP_UNHANDLED,
+                    "DW_DLE_MACRO_OP_UNHANDLED "
+                    "A miscount of ops_count "
+                    "So an internal libdwarf error");
+                return DW_DLV_ERROR;
+            }
+#endif /* 0 */
             macro_context->mc_macro_ops_count = opcount;
             macro_context->mc_ops_data_length = opslen;
             macro_context->mc_total_length = opslen +
@@ -587,7 +608,6 @@ dwarf_get_macro_defundef(Dwarf_Macro_Context macro_context,
         ress = _dwarf_extract_string_offset_via_str_offsets(dbg,
             mdata_copy,
             endptr,
-            DW_AT_macros, /*arbitrary, unused by called routine. */
             form1,
             macro_context->mc_cu_context,
             &offsettostr,
@@ -643,7 +663,8 @@ dwarf_get_macro_defundef(Dwarf_Macro_Context macro_context,
             &localstring, &lerr);
         if (resup != DW_DLV_OK) {
             if (resup == DW_DLV_ERROR) {
-                int myerrno = dwarf_errno(lerr);
+                Dwarf_Unsigned myerrno =
+                    (unsigned int)dwarf_errno(lerr);
                 if (myerrno == DW_DLE_NO_TIED_FILE_AVAILABLE) {
                     *macro_string =
                         (char *)"<DW_FORM_str_sup-no-tied_file>";
@@ -652,7 +673,7 @@ dwarf_get_macro_defundef(Dwarf_Macro_Context macro_context,
                     *macro_string =
                         (char *)"<Error: DW_FORM_str_sup-got-error>";
                 }
-                dwarf_dealloc(dbg,lerr,DW_DLA_ERROR);
+                dwarf_dealloc_error(dbg,lerr);
             } else {
                 *macro_string = "<DW_FORM_str_sup-no-entry>";
             }
@@ -706,8 +727,8 @@ static const char *
 construct_from_dir_and_name(const char *dir,
     const char *name)
 {
-    int truelen = 0;
-    char *final = 0;
+    size_t truelen = 0;
+    char  *final   = 0;
 
     /* Allow for NUL char and added /  */
     truelen = strlen(dir) + strlen(name) + 1 +1;
@@ -997,13 +1018,8 @@ read_operands_table(Dwarf_Macro_Context macro_context,
         /*  Compiler warning about unused opcode_number
             variable should be ignored. */
         Dwarf_Unsigned formcount = 0;
-#if 0   /* No need to actually read, just update pointer*/
-        UNUSEDARG Dwarf_Small opcode_number = 0;
-        READ_UNALIGNED_CK(dbg,opcode_number,Dwarf_Small,
-            macro_data,sizeof(Dwarf_Small),error,endptr);
-#endif /*0*/
-        macro_data += sizeof(Dwarf_Small);
 
+        macro_data += sizeof(Dwarf_Small);
         DECODE_LEB128_UWORD_CK(macro_data,formcount,
             dbg, error, endptr);
         cur_offset = (formcount+ macro_data) - section_base;
@@ -1020,11 +1036,11 @@ read_operands_table(Dwarf_Macro_Context macro_context,
     macro_context->mc_opcode_forms =  (struct Dwarf_Macro_Forms_s *)
         calloc(operand_table_count,
             sizeof(struct Dwarf_Macro_Forms_s));
-    macro_context->mc_opcode_count = operand_table_count;
     if (!macro_context->mc_opcode_forms) {
         _dwarf_error(dbg, error, DW_DLE_ALLOC_FAIL);
         return DW_DLV_ERROR;
     }
+    macro_context->mc_opcode_count = operand_table_count;
 
     curformentry = macro_context->mc_opcode_forms;
     for (i = 0; i < operand_table_count; ++i,++curformentry) {
@@ -1045,7 +1061,7 @@ read_operands_table(Dwarf_Macro_Context macro_context,
             dbg, error, endptr);
 
         curformentry->mf_code = opcode_number;
-        curformentry->mf_formcount = formcount;
+        curformentry->mf_formcount = (Dwarf_Small)formcount;
 
         cur_offset = (formcount+ macro_data) - section_base;
         if (cur_offset >= section_size) {
@@ -1320,8 +1336,10 @@ _dwarf_internal_macro_context_by_offset(Dwarf_Debug dbg,
 
     section_base = dbg->de_debug_macro.dss_data;
     section_size = dbg->de_debug_macro.dss_size;
+    /* guarding against overflow */
     /*  The '3'  ensures the header initial bytes present too. */
-    if ((3+macro_offset) >= section_size) {
+    if ((macro_offset >= section_size) ||
+        ((3+macro_offset) >= section_size) ) {
         dealloc_macro_srcfiles(srcfiles,srcfilescount);
         _dwarf_error(dbg, error, DW_DLE_MACRO_OFFSET_BAD);
         return DW_DLV_ERROR;
@@ -1329,7 +1347,6 @@ _dwarf_internal_macro_context_by_offset(Dwarf_Debug dbg,
     macro_header = macro_offset + section_base;
     macro_data = macro_header;
     section_end = section_base +section_size;
-
     macro_context = (Dwarf_Macro_Context)
         _dwarf_get_alloc(dbg,DW_DLA_MACRO_CONTEXT,1);
     if (!macro_context) {
@@ -1394,8 +1411,8 @@ _dwarf_internal_macro_context_by_offset(Dwarf_Debug dbg,
     macro_context->mc_macro_header = macro_header;
     macro_context->mc_section_offset = macro_offset;
     macro_context->mc_section_size = section_size;
-    macro_context->mc_version_number = version;
-    macro_context->mc_flags = flags;
+    macro_context->mc_version_number = (Dwarf_Half)version;
+    macro_context->mc_flags = (Dwarf_Small)flags;
     macro_context->mc_dbg = dbg;
     macro_context->mc_offset_size_flag =
         flags& MACRO_OFFSET_SIZE_FLAG?TRUE:FALSE;
@@ -1439,8 +1456,8 @@ _dwarf_internal_macro_context_by_offset(Dwarf_Debug dbg,
 
     macro_data += optablesize;
     macro_context->mc_macro_ops = macro_data;
-    macro_context->mc_macro_header_length =macro_data - macro_header;
-
+    macro_context->mc_macro_header_length =
+        (Dwarf_Half)(macro_data - macro_header);
     build_ops_array = FALSE;
     res = _dwarf_get_macro_ops_count_internal(macro_context,
         build_ops_array,
@@ -1592,10 +1609,11 @@ dwarf_get_macro_context_by_offset(Dwarf_Die cu_die,
 
 int dwarf_get_macro_section_name(Dwarf_Debug dbg,
     const char **sec_name_out,
-    UNUSEDARG Dwarf_Error *error)
+    Dwarf_Error *error)
 {
     struct Dwarf_Section_s *sec = 0;
 
+    CHECK_DBG(dbg,error,"dwarf_get_macro_section_name()");
     sec = &dbg->de_debug_macro;
     if (sec->dss_size == 0) {
         /* We don't have such a  section at all. */
@@ -1613,8 +1631,25 @@ dwarf_dealloc_macro_context(Dwarf_Macro_Context mc)
     if (!mc) {
         return;
     }
+    /*  Fixing coverity sccan CID 531842 and CID 531840.
+        Memory leak. The destructor would do this,
+        but coverity scan does not seem to track that.
+        so we do the free()s here and removed the
+        use of _dwarf_macro_destructor() from dwarf_alloc.c
+        And we will delete _dwarf_macro_destructor()
+        everywhere. */
     dbg = mc->mc_dbg;
-    /* See _dwarf_macro_destructor() here */
+    dealloc_macro_srcfiles(mc->mc_srcfiles, mc->mc_srcfiles_count);
+    mc->mc_srcfiles = 0;
+    mc->mc_srcfiles_count = 0;
+    free((void *)mc->mc_file_path);
+    mc->mc_file_path = 0;
+    free(mc->mc_ops);
+    mc->mc_ops = 0;
+    free(mc->mc_opcode_forms);
+    mc->mc_opcode_forms = 0;
+    /*  See _dwarf_macro_destructor() here, this
+        has a destructor.  */
     dwarf_dealloc(dbg,mc,DW_DLA_MACRO_CONTEXT);
 }
 
@@ -1628,38 +1663,3 @@ _dwarf_macro_constructor(Dwarf_Debug dbg, void *m)
     mc->mc_dbg = dbg;
     return DW_DLV_OK;
 }
-
-/*  Here we free various fields of Dwarf_Macro_Context.
-    The fields do not get dealloc'd.
-    If we had a separate destructor for hand-calling
-    (meaning when an error is detected during creation
-    of a  Dwarf_Macro_Context)
-    and one for calling by dwarf_dealloc() then
-    we could have the hand-calling dwarf_dealloc the fields
-    and the one called on the dealloc of a  Dwarf_Macro_Context
-    could leave the _dwarf_get_alloc() fields for for
-    normal dwarf_finish() cleanup.
-
-    But for now we share this destructor for both purposes
-    so no fields are _dwarf_get_alloc() and all are free-d
-    here..
-*/
-void
-_dwarf_macro_destructor(void *m)
-{
-    Dwarf_Macro_Context mc= (Dwarf_Macro_Context)m;
-
-    dealloc_macro_srcfiles(mc->mc_srcfiles, mc->mc_srcfiles_count);
-    mc->mc_srcfiles = 0;
-    mc->mc_srcfiles_count = 0;
-    free((void *)mc->mc_file_path);
-    mc->mc_file_path = 0;
-    free(mc->mc_ops);
-    mc->mc_ops = 0;
-    free(mc->mc_opcode_forms);
-    mc->mc_opcode_forms = 0;
-    memset(mc,0,sizeof(*mc));
-    /*  Just a recognizable sentinel.
-        For debugging.  No real meaning . */
-    mc->mc_sentinel = 0xdeadbeef;
-}
